#
# $Id: processor.rb 28 2005-01-13 00:51:17Z thomas $
#
#--
#
# xmlresume2x: converts an xml resume to various output formats
# Copyright (C) 2004 Thomas Leitner
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU
# General Public License as published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
# even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program; if not,
# write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#++
#

require 'xmlresume2x/wrapper'

module XMLResume2x

  # This class serves as container for processors and keywords. Therefore it defines the methods
  # which can be used in the configuration files.
  class ResumeProcessor

    # A hash of processors
    attr_reader :processors

    # A hash containing the assignments of XML elements to processors
    attr_reader :assignments

    # The language used for this ResumeProcessor
    attr_accessor :lang

    def initialize( lang )
      @lang = lang
      @keywords = Hash.new
      @processors = Hash.new( Proc.new {|e, options| get_children_value( e, options ) } )
      @assignments = Hash.new
      processor( :textItem => 'REXMLTextProcessor' ) {|e, options| get_value( e.value, options ) }
    end

    # Loads the configuration defined by the given string +src+.
    def load_config( src, filename = '(eval)' )
      instance_eval( src, filename )
    end

    # Adds a processor which should be defined using a block. +name+ is the name of the processor.
    def add_processor( name, &block )
      raise ArgumentError, "No block given" unless block_given?
      @processors[name.to_s] = block
    end

    # Assigns elements to processors. +assignment+ has to be a Hash where the keys are XML resume
    # element names and the values are processor names. If a block is supplied, then the XML resume
    # elements are processed by the given block.
    def processor( assignment, &block )
      raise ArgumentError, "Argument has to be a Hash" unless Hash === assignment
      assignment.each do |key, value|
        add_processor( value, &block ) if block_given?
        case key
        when Array
          key.each {|name| @assignments[name.to_s] = value }
        when String
          @assignments[key.to_s] = value
        when Symbol
          @assignments[key] = value
        else
          raise ArgumentError, "Key for assignment has to be Array or String or Symbol"
        end
      end
    end

    # Calls the processor +name+ on the element +item+ and optionally with +options+.
    def call_processor( name, item, options = {} )
      proc = @processors[name]
      if proc.arity == 1
        proc.call( item )
      else
        proc.call( item, options )
      end
    end

    # Defines or retrieves a keyword. If +arg+ is a Hash, then the keywords in this Hash are added.
    # If +arg+ is a Symbol, then the specified keyword is retrieved. The parameter +processed+
    # specifies if the keyword should be processed as any other text.
    def keyword( arg, processed = true )
      case arg
      when Symbol
        processed ? get_value( @keywords[arg] ) : @keywords[arg]
      when Hash
        @keywords.update( arg )
      else
        raise ArgumentError, "Argument has to be a Symbol or a Hash"
      end
    end

    # Returns the value of +item+ after being processed by the assigned processor.
    def get_value( item, options = {} )
      name, item = case item
                   when ElementWrapper
                     options = item.options.merge( options )
                     [item.__element.name, item]
                   when REXML::Element
                     [item.name, ElementWrapper.new( item, self, options )]
                   when REXML::Text
                     [:textItem, item]
                   when String
                     [:text, item]
                   when NilClass
                     return nil
                   else
                     raise ArgumentError, "Not a valid argument class (#{item.class})"
                   end
      call_processor( @assignments[name], item, options )
    end

    # Returns the values of the processed children of +element+.
    # *Returns*:: an array with all the values or if the <tt>:to_s</tt>
    #             option is set, the array converted to a string, i.e. <tt>array.to_s</tt>
    def get_children_value( element, options = {} )
      arr = []
      new_options = element.options.merge( options )
      element.__element.each_child {|child| arr << get_value( child, new_options ) }
      new_options[:to_s] ? arr.to_s : arr
    end

  end

end
